"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@rtcts/utils");
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const prettier_package_json = __importStar(require("prettier-package-json"));
const field_order = [
    "context",
    "entry",
    "dll_entry",
    "lib_entry",
    "modules",
    "output",
    "post_css_plugins_path",
    "sync_project_deps",
    "is_application",
    "is_back_end",
    "has_dll",
    "has_browser_lib",
    "load_on_demand",
    "has_node_lib",
    "publish_in_git",
    "docker_org_name",
];
class VersionableConfig {
    constructor(config_path) {
        this.config_path = path.resolve(process.cwd(), "package.json");
        if (utils_1.isString(config_path)) {
            this.config_path = config_path;
        }
    }
    get config() {
        return this.pkg.rearguard;
    }
    set config(fields) {
        const unsorted_rearguard = { ...this.pkg.rearguard, ...fields };
        const sorted_rearguard = {};
        for (const key of field_order) {
            if (unsorted_rearguard.hasOwnProperty(key)) {
                sorted_rearguard[key] = unsorted_rearguard[key];
            }
        }
        const pkg = JSON.parse(prettier_package_json.format(this.pkg).trim());
        pkg.rearguard = sorted_rearguard;
        fs.writeFileSync(this.config_path, JSON.stringify(pkg, null, 2).trim(), { encoding: "utf-8" });
    }
    get pkg() {
        if (fs.existsSync(this.config_path)) {
            try {
                const pkg = JSON.parse(fs.readFileSync(this.config_path).toString());
                if (!utils_1.isObject(pkg.rearguard)) {
                    this.pkg = { rearguard: {} };
                    console.log(chalk_1.default.greenBright(`[ VERSIONABLE-CONFIG ][ CREATED ] FIELD 'rearguard' IN: ${this.config_path};`));
                    console.log("");
                    return this.pkg;
                }
                return pkg;
            }
            catch (error) {
                console.error(error);
                return {};
            }
        }
        else {
            console.trace(chalk_1.default.bold.red(`[ ${this.constructor.name} ][ ERROR ][ You haven't package.json here: ${this.config_path} ]`));
            process.exit(1);
            return {};
        }
    }
    set pkg(fields) {
        if (fs.existsSync(this.config_path)) {
            try {
                const pkg = JSON.parse(fs.readFileSync(this.config_path).toString());
                fs.writeFileSync(this.config_path, prettier_package_json.format({ ...pkg, ...fields }).trim());
            }
            catch (error) {
                console.error(error);
                process.exit(1);
            }
        }
        else {
            console.trace(chalk_1.default.bold.red(`[ ${this.constructor.name} ][ ERROR ][ You haven't package.json here: ${this.config_path} ]`));
            process.exit(1);
        }
    }
    order_config_fields() {
        this.config = this.config;
    }
}
exports.VersionableConfig = VersionableConfig;
//# sourceMappingURL=VersionableConfig.js.map